/**
 * TEXAS INSTRUMENTS TEXT FILE LICENSE
 * 
 * 
 * Copyright (c) 2014 - 2017 Texas Instruments Incorporated
 * 
 * All rights reserved not granted herein.
 * 
 * Limited License.  
 * 
 * Texas Instruments Incorporated grants a world-wide, royalty-free, non-exclusive license under copyrights and patents it now or hereafter owns or controls to make, have made, use, import, offer to sell and sell ("Utilize") this software subject to the terms herein.  With respect to the foregoing patent license, such license is granted  solely to the extent that any such patent is necessary to Utilize the software alone.  The patent license shall not apply to any combinations which include this software, other than combinations with devices manufactured by or for TI ("TI Devices").  No hardware patent is licensed hereunder.
 * 
 * Redistributions must preserve existing copyright notices and reproduce this license (including the above copyright notice and the disclaimer and (if applicable) source code license limitations below) in the documentation and/or other materials provided with the distribution
 * 
 * Redistribution and use in binary form, without modification, are permitted provided that the following conditions are met:
 * 
 * *	No reverse engineering, decompilation, or disassembly of this software is permitted with respect to any software provided in binary form.
 * 
 * *	any redistribution and use are licensed by TI for use only with TI Devices.
 * 
 * *	Nothing shall obligate TI to provide you with source code for the software licensed and provided to you in object code.
 * 
 * If software source code is provided to you, modification and redistribution of the source code are permitted provided that the following conditions are met:
 * 
 * *	any redistribution and use of the source code, including any resulting derivative works, are licensed by TI for use only with TI Devices.
 * 
 * *	any redistribution and use of any object code compiled from the source code and any resulting derivative works, are licensed by TI for use only with TI Devices.
 * 
 * Neither the name of Texas Instruments Incorporated nor the names of its suppliers may be used to endorse or promote products derived from this software without specific prior written permission.
 * 
 * DISCLAIMER.
 * 
 * THIS SOFTWARE IS PROVIDED BY TI AND TI'S LICENSORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TI'S LICENSORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * \file BSL432_Command_Interpreter.h
 * \brief Header file for the BSL Command Interpreter functions.
 */

#ifndef BSL432_COMMAND_INTERPRETER_H_
#define BSL432_COMMAND_INTERPRETER_H_

#include <stdint.h>

extern uint8_t *BSL432_ReceiveBuffer;
extern uint8_t *BSL432_SendBuffer;
extern uint32_t BSL432_ReceiveBufferSize;
extern uint32_t BSL432_SendBufferSize;

extern uint32_t BSL432_BSL_PARAMS;

/**
 * Interprets the command contained in the data buffer and
 * calls the appropriate BSL API function
 */
extern void BSL432_CI_interpretCommand();

/**
 * Calls the API to write a data block to a given memory location
 */
extern void BSL432_CI_receiveDataBlock(uint32_t addr, uint8_t *data);

/**
 * Calls the API to write a data block to a given memory location
 */
extern void BSL432_CI_receiveDataBlock32(uint32_t addr, uint8_t *data);

/**
 * Calls the API to unlock the BSL
 */
extern void BSL432_CI_receivePassword();

/**
 * Calls the API to perform a CRC check over a given addr and length of data
 */
extern void BSL432_CI_CRCCheck(uint32_t addr, uint32_t length);

/**
 * Fills the SendBuffer array with bytes from the given parameters.
 * Sends the data by calling the PI, or sends an error.
 */
extern void BSL432_CI_sendDataBlock(uint32_t addr, uint32_t length);

/**
 * Sends a reply message with attached information.
 */
extern void BSL432_CI_sendMessage(uint8_t message);

#endif /* BSL432_COMMAND_INTERPRETER_H_ */
