/**
 * TEXAS INSTRUMENTS TEXT FILE LICENSE
 * 
 * 
 * Copyright (c) 2014 - 2017 Texas Instruments Incorporated
 * 
 * All rights reserved not granted herein.
 * 
 * Limited License.  
 * 
 * Texas Instruments Incorporated grants a world-wide, royalty-free, non-exclusive license under copyrights and patents it now or hereafter owns or controls to make, have made, use, import, offer to sell and sell ("Utilize") this software subject to the terms herein.  With respect to the foregoing patent license, such license is granted  solely to the extent that any such patent is necessary to Utilize the software alone.  The patent license shall not apply to any combinations which include this software, other than combinations with devices manufactured by or for TI ("TI Devices").  No hardware patent is licensed hereunder.
 * 
 * Redistributions must preserve existing copyright notices and reproduce this license (including the above copyright notice and the disclaimer and (if applicable) source code license limitations below) in the documentation and/or other materials provided with the distribution
 * 
 * Redistribution and use in binary form, without modification, are permitted provided that the following conditions are met:
 * 
 * *	No reverse engineering, decompilation, or disassembly of this software is permitted with respect to any software provided in binary form.
 * 
 * *	any redistribution and use are licensed by TI for use only with TI Devices.
 * 
 * *	Nothing shall obligate TI to provide you with source code for the software licensed and provided to you in object code.
 * 
 * If software source code is provided to you, modification and redistribution of the source code are permitted provided that the following conditions are met:
 * 
 * *	any redistribution and use of the source code, including any resulting derivative works, are licensed by TI for use only with TI Devices.
 * 
 * *	any redistribution and use of any object code compiled from the source code and any resulting derivative works, are licensed by TI for use only with TI Devices.
 * 
 * Neither the name of Texas Instruments Incorporated nor the names of its suppliers may be used to endorse or promote products derived from this software without specific prior written permission.
 * 
 * DISCLAIMER.
 * 
 * THIS SOFTWARE IS PROVIDED BY TI AND TI'S LICENSORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TI'S LICENSORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * \file BSL432_Peripheral_Interface.h
 * \brief Header file for the BSL Peripheral Interface functions.
 */

#ifndef BSL432_PERIPHERAL_INTERFACE_H_
#define BSL432_PERIPHERAL_INTERFACE_H_

#include <stdint.h>

#define BSL432_PI_COMMAND_UPPER    0x50
#define BSL432_HEADER_INCORRECT    (BSL432_PI_COMMAND_UPPER + 0x01)
#define BSL432_CHECKSUM_INCORRECT  (BSL432_PI_COMMAND_UPPER + 0x02)
#define BSL432_PACKET_SIZE_ZERO    (BSL432_PI_COMMAND_UPPER + 0x03)
#define BSL432_PACKET_SIZE_TOO_BIG (BSL432_PI_COMMAND_UPPER + 0x04)
#define BSL432_UNKNOWN_ERROR       (BSL432_PI_COMMAND_UPPER + 0x05)
#define BSL432_UNKNOWN_BAUD_RATE   (BSL432_PI_COMMAND_UPPER + 0x06)

#define BSL432_MAX_BUFFER_SIZE 262

/* Number of clocks per UART symbol for automatic baudrate detection
 * 9600, 19200, 38400, 57600, 115200 baud; each +/-10%
 */
#define BSL432_UART_BAUDRATE_CPS_9600_MAX    2750
#define BSL432_UART_BAUDRATE_CPS_9600_MIN    2250
#define BSL432_UART_BAUDRATE_CPS_19200_MAX   1375
#define BSL432_UART_BAUDRATE_CPS_19200_MIN   1125
#define BSL432_UART_BAUDRATE_CPS_38400_MAX   688
#define BSL432_UART_BAUDRATE_CPS_38400_MIN   563
#define BSL432_UART_BAUDRATE_CPS_57600_MAX   458
#define BSL432_UART_BAUDRATE_CPS_57600_MIN   375
#define BSL432_UART_BAUDRATE_CPS_115200_MAX  229
#define BSL432_UART_BAUDRATE_CPS_115200_MIN  187

/*
 * Additional definition as the new header file for MSP432P4111 family
 * using the CMSIS style.
 * The "UCTXIE" and "UCRXIE" are still used for the compatibility
 * with the code for MSP432410x.
 */

#define UCTXIE EUSCI_A_IE_RXIE
#define UCRXIE EUSCI_A_IE_TXIE

typedef enum
{
    BSL432_UARTBaudRatesAuto   = 0x00,
    BSL432_UARTBaudRates4800   = 0x01,
    BSL432_UARTBaudRates9600   = 0x02,
    BSL432_UARTBaudRates19200  = 0x03,
    BSL432_UARTBaudRates38400  = 0x04,
    BSL432_UARTBaudRates57600  = 0x05,
    BSL432_UARTBaudRates115200 = 0x06,
} BSL432_UARTBaudRates;

typedef enum
{
    Flags_rxPacketOngoing    = 0x00,
    Flags_data_received      = 0x01,
    Flags_rxErrorRecoverable = 0x02,
    Flags_rxErrorReinit      = 0x04,
    Flags_rxErrorFatal       = 0x08,
    Flags_PIdataReceived     = 0x10,
} BSL432_eventFlags;

typedef enum
{
    BSL432_AP_Auto = 0x07,
    BSL432_AP_UART = 0x06,
    BSL432_AP_SPI  = 0x05,
    BSL432_AP_I2C  = 0x04,
} BSL432_ActivePeripherals;

typedef enum
{
    IRQ_idle = 0x0,
    IRQ_receiving = 0x1,
    IRQ_readytosend = 0x2,
    IRQ_sending = 0x3,
} BSL432_IRQ_states;

/* Function pointers to the current peripherals functions */
extern void (*BSL432_PI_sendByte)(uint8_t);
extern uint8_t (*BSL432_PI_receiveByte)();
extern void (*BSL432_PI_sendData)(uint32_t);
extern BSL432_eventFlags (*BSL432_PI_receivePacket)();

extern BSL432_ActivePeripherals BSL432_ActivePeripheral;
extern uint32_t BSL432_UART_MODULE;
extern uint32_t BSL432_SPI_MODULE;
extern uint32_t BSL432_I2C_MODULE;
extern uint16_t BSL432_I2C_SLAVE_ADDRESS;

extern volatile BSL432_IRQ_states BSL432_IRQ_state;

/**
 * PI initialization routine configures all peripherals.
 */
extern void BSL432_PI_init();

/**
 * PI interface configuration sets up the used peripheral modules
 */
extern void BSL432_PI_configureInterfaces();

/**
 * PI routine that scans all active interfaces for received data
 */
extern void BSL432_PI_detectInterface();

/**
 * PI initialization routine for passive (RX only) UART auto baudrate detection.
 */
extern void BSL432_PI_initUARTdetect();

/**
 * PI auto baud rate detection
 */
extern void BSL432_PI_UARTdetect();

/**
 * PI initialization routine for UART.
 */
extern void BSL432_PI_initUART(BSL432_UARTBaudRates);

/**
 * PI initialization routine for SPI.
 */
extern void BSL432_PI_initSPI();

/**
 * PI initialization routine for I2C.
 */
extern void BSL432_PI_initI2C();

/**
 * PI disable function for UART
 */
extern void BSL432_PI_disableUART();

/**
 * PI disable function for SPI
 */
extern void BSL432_PI_disableSPI();

/**
 * PI disable function for I2C
 */
extern void BSL432_PI_disableI2C();

/**
 * PI initialization routine for BSL timeout using Timer_A
 */
extern void BSL432_PI_initTimeout();

/**
 * Processes PI commands
 */
extern void BSL432_PI_interpretPICommand();

/**
 * Returns the maximum data buffer size for the current PI
 */
extern uint32_t BSL432_PI_getBufferSize();

/**
 * Verifies the data in the receive buffer against a given checksum
 */
extern uint32_t BSL432_PI_verifyData(uint32_t checksum);

/**
 * Reads and entire packet from the peripheral, verifies it,
 * and sends it to the core to be interpreted
 */
extern BSL432_eventFlags BSL432_PI_receivePacketUART();

/**
 * Sends a single byte via UART peripheral
 */
extern void BSL432_PI_sendByteUART(uint8_t data);

/**
 * Sends data in the data buffer via UART
 */
extern void BSL432_PI_sendDataUART(uint32_t size);

/**
 * Reads and entire packet from the SPI peripheral, verifies it,
 * and sends it to the core to be interpreted
 */
extern BSL432_eventFlags BSL432_PI_receivePacketSPI();

/**
 * Sends a single byte via SPI peripheral
 */
extern void BSL432_PI_sendByteSPI(uint8_t data);

/**
 * Sends a single byte via SPI peripheral directly without using the send buffer
 */
extern void BSL432_PI_sendByteImmediatelySPI(uint8_t data);

/**
 * Sends data in the data buffer via SPI
 */
extern void BSL432_PI_sendDataSPI(uint32_t size);

/**
 * Handles the receiving and sending of bytes via I2C.
 */
extern BSL432_eventFlags BSL432_PI_receivePacketI2C();

/**
 * Places a single byte to be sent into the transmit buffer
 */
extern void BSL432_PI_sendByteI2C(uint8_t data);

/**
 * Sends a single byte via I2C peripheral directly without using the send buffer
 */
extern void BSL432_PI_sendByteImmediatelyI2C(uint8_t data);

/**
 * Sends data in the data buffer via I2C
 */
extern void BSL432_PI_sendDataI2C(uint32_t size);

/**
 * Interrupt handler for the first byte received via SPI
 */
extern void BSL432_PI_IRQ_SPIdetect(void);

/**
 * Interrupt handler for RX operation via SPI
 */
extern void BSL432_PI_IRQ_SPI_RX(void);

/**
 * Interrupt handler for TX operation via SPI
 */
extern void BSL432_PI_IRQ_SPI_TX(void);

/**
 * Interrupt handler for RX operation via UART
 */
extern void BSL432_PI_IRQ_UART(void);

/**
 * Interrupt handler for the first byte received via I2C
 */
extern void BSL432_PI_IRQ_I2Cdetect(void);

/**
 * Interrupt handler for RX/TX operation via I2C
 */
extern void BSL432_PI_IRQ_I2C(void);

#endif /* BSL432_PERIPHERAL_INTERFACE_H_ */
