/**
 * \file BSL432_device_file.h
 * \brief The BSL device file contains device specific settings for the BSL.
 */

/*
 * Changelog
 * 2015-06-03  0004  MG  Adding TLV information for MSP432P401R
 * 2015-07-26  000B  MG  Modifying for MSP432P4111
 * 2017-11-28  0007  FS  Add different header file for MSP432P4111, MSP432P411V, and MSP432P411Y.
 */

#ifndef BSL432_DEVICE_FILE_H_
#define BSL432_DEVICE_FILE_H_

#if defined (__MSP432P4111__)
   #include "msp432p4111.h"
#elif defined (__MSP432P411V__)
   #include "msp432p411v.h"
#elif defined (__MSP432P411Y__)
   #include "msp432p411y.h"
#else
   #error "Device not supported by this project"
#endif

#include "driverlib.h"

#define BSL432_VERSION  {0x00, 0x0D}  // Build ID

#define BSL432_RAM_BSL_PARAMS     0x200000e8
#define BSL432_SECURE_RAM_START   (BSL432_RAM_BSL_PARAMS + 4)  // Do not overwrite the BSL_PARAMS
#define BSL432_SECURE_RAM_LENGTH  (0x20001FC4 - BSL432_SECURE_RAM_START)  // Clear up to the stack pointer

#define BSL432_INTERRUPT_VECTOR_START  0x0
#define BSL432_INTERRUPT_VECOTR_END    0xFF

#define BSL432_RAM_START  0x20000000

#define BSL432_TIMEOUT 32768  /** BSL timeout in ~1/3276 seconds */

/* Flash mailbox */
#define BSL432_FLASHMAILBOXSTART  0x200000
#define MB_ADDR_START  (*((volatile uint32_t *) (BSL432_FLASHMAILBOXSTART + 0x0)))
#define MB_ADDR_CMD    (*((volatile uint32_t *) (BSL432_FLASHMAILBOXSTART + 0x4)))
#define MB_ADDR_FRPW   (*((volatile uint32_t *) (BSL432_FLASHMAILBOXSTART + 0x270)))
#define MB_ADDR_FRACK  (*((volatile uint32_t *) (BSL432_FLASHMAILBOXSTART + 0x280)))
#define MB_ADDR_END    (*((volatile uint32_t *) (BSL432_FLASHMAILBOXSTART + 0x28C)))

#define MB_START_SIG  0x00115ACF6  // Mailbox start signature
#define MB_FACTORY_RESET_PARAMS  0x00010000
#define MB_END_SIG  0x0011E11D  // Mailbox end signature


/* TLV BSL entries */
#define BSL432_TLV_BSL_CONFIGURATION_TAG  0x0F
#define BSL432_TLV_INSTANCE_BSL_PERIPHERAL_INTERFACE_SELECTION  0

/* BSL_PER_IF_SEL */
#define BSL432_BSL_PER_IF_SEL_UART_INST     (uint8_t)(((BSL432_TLV_BSL_Configuration->BSLPeripheralInterfaceSelection) >> 0) & 0b1111)
#define BSL432_BSL_PER_IF_SEL_UART_MOD      (uint8_t)(((BSL432_TLV_BSL_Configuration->BSLPeripheralInterfaceSelection) >> 4) & 0b11)
#define BSL432_BSL_PER_IF_SEL_UART_MUX      (uint8_t)(((BSL432_TLV_BSL_Configuration->BSLPeripheralInterfaceSelection) >> 7) & 0b1)
#define BSL432_BSL_PER_IF_SEL_SPI_MOD       (uint8_t)(((BSL432_TLV_BSL_Configuration->BSLPeripheralInterfaceSelection) >> 12) & 0b11)
#define BSL432_BSL_PER_IF_SEL_SPI_INST      (uint8_t)(((BSL432_TLV_BSL_Configuration->BSLPeripheralInterfaceSelection) >> 8) & 0b1111)
#define BSL432_BSL_PER_IF_SEL_SPI_MUX       (uint8_t)(((BSL432_TLV_BSL_Configuration->BSLPeripheralInterfaceSelection) >> 15) & 0b1)
#define BSL432_BSL_PER_IF_SEL_I2C_INST      (uint8_t)(((BSL432_TLV_BSL_Configuration->BSLPeripheralInterfaceSelection) >> 16) & 0b1111)
#define BSL432_BSL_PER_IF_SEL_I2C_MOD       (uint8_t)(((BSL432_TLV_BSL_Configuration->BSLPeripheralInterfaceSelection) >> 20) & 0b11)
#define BSL432_BSL_PER_IF_SEL_I2C_MUX       (uint8_t)(((BSL432_TLV_BSL_Configuration->BSLPeripheralInterfaceSelection) >> 23) & 0b1)

/* BSL_PORTCNF_UART */
#define BSL432_BSL_PORTCNF_UART_UART_PORT   (uint8_t)(((BSL432_TLV_BSL_Configuration->BSLPortInterfaceConfigurationUART) >> 0) & 0b1111)
#define BSL432_BSL_PORTCNF_UART_BITPOS_RXD  (uint8_t)(((BSL432_TLV_BSL_Configuration->BSLPortInterfaceConfigurationUART) >> 4) & 0b111)
#define BSL432_BSL_PORTCNF_UART_BITPOS_TXD  (uint8_t)(((BSL432_TLV_BSL_Configuration->BSLPortInterfaceConfigurationUART) >> 7) & 0b111)
#define BSL432_BSL_PORTCNF_UART_PSEL0_RXD   (uint8_t)(((BSL432_TLV_BSL_Configuration->BSLPortInterfaceConfigurationUART) >> 16) & 0b1)
#define BSL432_BSL_PORTCNF_UART_PSEL0_TXD   (uint8_t)(((BSL432_TLV_BSL_Configuration->BSLPortInterfaceConfigurationUART) >> 17) & 0b1)
#define BSL432_BSL_PORTCNF_UART_PSEL1_RXD   (uint8_t)(((BSL432_TLV_BSL_Configuration->BSLPortInterfaceConfigurationUART) >> 24) & 0b1)
#define BSL432_BSL_PORTCNF_UART_PSEL1_TXD   (uint8_t)(((BSL432_TLV_BSL_Configuration->BSLPortInterfaceConfigurationUART) >> 25) & 0b1)

/* BSL_PORTCNF_SPI */
#define BSL432_BSL_PORTCNF_SPI_SPI_PORT     (uint8_t)(((BSL432_TLV_BSL_Configuration->BSLPortInterfaceConfigurationSPI) >> 0) & 0b1111)
#define BSL432_BSL_PORTCNF_SPI_BITPOS_SOMI  (uint8_t)(((BSL432_TLV_BSL_Configuration->BSLPortInterfaceConfigurationSPI) >> 4) & 0b111)
#define BSL432_BSL_PORTCNF_SPI_BITPOS_CLK   (uint8_t)(((BSL432_TLV_BSL_Configuration->BSLPortInterfaceConfigurationSPI) >> 10) & 0b111)
#define BSL432_BSL_PORTCNF_SPI_BITPOS_SIMO  (uint8_t)(((BSL432_TLV_BSL_Configuration->BSLPortInterfaceConfigurationSPI) >> 7) & 0b111)
#define BSL432_BSL_PORTCNF_SPI_BITPOS_STE   (uint8_t)(((BSL432_TLV_BSL_Configuration->BSLPortInterfaceConfigurationSPI) >> 13) & 0b111)
#define BSL432_BSL_PORTCNF_SPI_PSEL0_SOMI   (uint8_t)(((BSL432_TLV_BSL_Configuration->BSLPortInterfaceConfigurationSPI) >> 16) & 0b1)
#define BSL432_BSL_PORTCNF_SPI_PSEL0_SIMO   (uint8_t)(((BSL432_TLV_BSL_Configuration->BSLPortInterfaceConfigurationSPI) >> 17) & 0b1)
#define BSL432_BSL_PORTCNF_SPI_PSEL0_CLK    (uint8_t)(((BSL432_TLV_BSL_Configuration->BSLPortInterfaceConfigurationSPI) >> 18) & 0b1)
#define BSL432_BSL_PORTCNF_SPI_PSEL0_STE    (uint8_t)(((BSL432_TLV_BSL_Configuration->BSLPortInterfaceConfigurationSPI) >> 19) & 0b1)
#define BSL432_BSL_PORTCNF_SPI_PSEL1_SOMI   (uint8_t)(((BSL432_TLV_BSL_Configuration->BSLPortInterfaceConfigurationSPI) >> 24) & 0b1)
#define BSL432_BSL_PORTCNF_SPI_PSEL1_SIMO   (uint8_t)(((BSL432_TLV_BSL_Configuration->BSLPortInterfaceConfigurationSPI) >> 25) & 0b1)
#define BSL432_BSL_PORTCNF_SPI_PSEL1_CLK    (uint8_t)(((BSL432_TLV_BSL_Configuration->BSLPortInterfaceConfigurationSPI) >> 26) & 0b1)
#define BSL432_BSL_PORTCNF_SPI_PSEL1_STE    (uint8_t)(((BSL432_TLV_BSL_Configuration->BSLPortInterfaceConfigurationSPI) >> 27) & 0b1)

/* BSL_PORTCNF_I2C */
#define BSL432_BSL_PORTCNF_I2C_I2C_PORT     (uint8_t)(((BSL432_TLV_BSL_Configuration->BSLPortInterfaceConfigurationI2C) >> 0) & 0b1111)
#define BSL432_BSL_PORTCNF_I2C_BITPOS_SCL   (uint8_t)(((BSL432_TLV_BSL_Configuration->BSLPortInterfaceConfigurationI2C) >> 4) & 0b111)
#define BSL432_BSL_PORTCNF_I2C_BITPOS_SDA   (uint8_t)(((BSL432_TLV_BSL_Configuration->BSLPortInterfaceConfigurationI2C) >> 7) & 0b111)
#define BSL432_BSL_PORTCNF_I2C_PSEL0_SCL    (uint8_t)(((BSL432_TLV_BSL_Configuration->BSLPortInterfaceConfigurationI2C) >> 16) & 0b1)
#define BSL432_BSL_PORTCNF_I2C_PSEL0_SDA    (uint8_t)(((BSL432_TLV_BSL_Configuration->BSLPortInterfaceConfigurationI2C) >> 17) & 0b1)
#define BSL432_BSL_PORTCNF_I2C_PSEL1_SCL    (uint8_t)(((BSL432_TLV_BSL_Configuration->BSLPortInterfaceConfigurationI2C) >> 24) & 0b1)
#define BSL432_BSL_PORTCNF_I2C_PSEL1_SDA    (uint8_t)(((BSL432_TLV_BSL_Configuration->BSLPortInterfaceConfigurationI2C) >> 25) & 0b1)

#endif /* BSL432_DEVICE_FILE_H_ */
